using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using DatabaseObjectSearcher;
using System.Runtime.InteropServices;
using Microsoft.SqlServer.Management.Smo;
using DatabaseObjectSearcherUI.Controls;


namespace DatabaseObjectSearcherUI
{
    [Guid("4c410c93-d66b-495a-9de2-99d5bde4a3b9")]
    public partial class SearchToolWindow : UserControl
    {
        DatabaseObjectSearcher.DatabaseObjectSearcher searcher; 

        public SearchToolWindow()
        {
            InitializeComponent();
            this.GotFocus += new EventHandler(SearchToolWindow_GotFocus);
            searchText.GotFocus += new EventHandler(SearchToolWindow_GotFocus);
            searchText.TextChanged += new EventHandler(searchText_TextChanged);
        }

        void searchText_TextChanged(object sender, EventArgs e)
        {
            resultList.Items.Clear();
            
            if (searchText.Text.Length > 1 && searcher != null)
            {
                List<DatabaseSearchResult> matches =searcher.FindMatchingObjects(searchText.Text);
                foreach (DatabaseSearchResult item in matches)
                {
                    // HAAAAACK
                    if (item.Result is StoredProcedure){
                        StoredProcedure sp = item.Result as StoredProcedure;
                        ImageListItem listItem = new ImageListItem(sp.Name + " - procedure - " + sp.Parent.Name, 1, false);
                        listItem.Tag = item;
                        resultList.Items.Add(listItem);
                    }
                    else if (item.Result is Table){
                        Table t = item.Result as Table;
                        ImageListItem listItem = new ImageListItem(t.Name + " - table - " + t.Parent.Name, 2, false);
                        listItem.Tag = item;
                        resultList.Items.Add(listItem);
                    }
                    else if (item.Result is Microsoft.SqlServer.Management.Smo.View){
                        Microsoft.SqlServer.Management.Smo.View v = item.Result as Microsoft.SqlServer.Management.Smo.View;
                        ImageListItem listItem = new ImageListItem(v.Name + " - view - " + v.Parent.Name , 3, false);
                        listItem.Tag = item;
                        resultList.Items.Add(listItem);
                    }
                    else if (item.Result is Database){
                        Database t = item.Result as Database;
                        ImageListItem listItem = new ImageListItem(t.Name + " - database", 0, false);
                        listItem.Tag = item;
                        resultList.Items.Add(listItem);
                    }
                    
                }
            }
        }

        void SearchToolWindow_GotFocus(object sender, EventArgs e)
        {
           searcher = SearcherController.Current.GetSearcher();
        }

        private void resultList_DoubleClick(object sender, EventArgs e)
        {
            SelectCurrentItemInObjectExplorer();
        }

        private void searchText_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Down || e.KeyCode == Keys.Enter)
            {
                if (resultList.Items != null && resultList.Items.Count > 0)
                {
                    resultList.SelectedIndex = 0;
                    resultList.Focus();
                }
            }
        }

        private void resultList_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Enter)
            {
                SelectCurrentItemInObjectExplorer();
            }
        }

        private void SelectCurrentItemInObjectExplorer()
        {
            if (resultList.SelectedItem != null)
            {
                DatabaseSearchResult selected = ((ImageListItem)resultList.SelectedItem).Tag as DatabaseSearchResult;
                if (selected != null)
                {
                    SearcherController.Current.SelectSMOObject(selected.Result, selected.Connection);
                }
            }
        }

    }
}
